const fs = require('node:fs');

const formatMentions = text => {
    const matcher = /@(\w+)@([a-zA-Z\-_0-9\.]+)/gm;
    return text.replace(matcher, (match, username, domain) => {
        return `[${match}](https://${domain}/@${username})`;
    });
}

const extractAndFormatHashtags = note => {
    const matcher = /#(\w+)/gm;
    const hashtags = new Set();
    note.text = note.text.replace(matcher, (match, hashtag) => {
        hashtags.add(hashtag.toLowerCase());
        return `[${match}](/tags/${hashtag.toLowerCase()})`;
    });
    return Array.from(hashtags);
}

module.exports = function() {
    const hashtags = {};

    const notes = JSON.parse(fs.readFileSync('./src/_data/_notes.json', 'utf-8'))
        .filter(note => note.text !== null && note.visibility === "public")
        .map(note => {
            extractAndFormatHashtags(note).forEach(hashtag => {
                if (!hashtags[hashtag]) {
                    hashtags[hashtag] = [];
                }
                hashtags[hashtag].push(note);
            });
    
            return {
                ...note,
                text: formatMentions(note.text),
                createdAt: new Date(note.createdAt),
                replies: []
            }
        });

    
    notes.forEach(note => {
        if (note.replyId) {
            notes.find(replied => replied.id === note.replyId)?.replies.push(note);
        }
    });
    
    return {
        notes,
        hashtags: Object.keys(hashtags).map(tag => ({
            tag,
            notes: hashtags[tag]
        }))
    };
}